@@ -358,33 +358,70 @@ int is_msg_for_me(snac *snac, const xs_dict *c_msg)
 }
 
 
-void process_tags(snac *snac, const char *content, d_char **n_content, d_char **tag)
+void process_tags(snac *snac, const char *content, xs_str **n_content, xs_list **tag)
 /* parses mentions and tags from content */
 {
-    d_char *nc = xs_str_new(NULL);
-    d_char *tl = xs_list_new();
+    xs_str *nc  = xs_str_new(NULL);
+    xs_list *tl = *tag;
     xs *split;
-    char *p, *v;
+    xs_list *p;
+    xs_val *v;
     int n = 0;
 
-    split = xs_regex_split(content, "(@[A-Za-z0-9_]+@[A-Za-z0-9\\.-]+|&#[0-9]+;|#[^ ,\\.:;<]+)");
+    /* create a default server for incomplete mentions */
+    xs *def_srv = NULL;
+
+    if (xs_list_len(tl)) {
+        /* if there are any mentions, get the server from
+           the first one, which is the inReplyTo author */
+        p = tl;
+        while (xs_list_iter(&p, &v)) {
+            const char *type = xs_dict_get(v, "type");
+            const char *name = xs_dict_get(v, "name");
+
+            if (type && name && strcmp(type, "Mention") == 0) {
+                xs *l = xs_split(name, "@");
+
+                def_srv = xs_dup(xs_list_get(l, -1));
+
+                break;
+            }
+        }
+    }
+
+    if (xs_is_null(def_srv))
+        /* use this same server */
+        def_srv = xs_dup(xs_dict_get(srv_config, "host"));
+
+    split = xs_regex_split(content, "(@[A-Za-z0-9_]+(@[A-Za-z0-9\\.-]+)?|&#[0-9]+;|#[^ ,\\.:;<]+)");
 
     p = split;
     while (xs_list_iter(&p, &v)) {
         if ((n & 0x1)) {
             if (*v == '@') {
+                xs *link = NULL;
+                xs *wuid = NULL;
+
+                if (strchr(v + 1, '@') == NULL) {
+                    /* only one @? it's a dumb Mastodon-like mention
+                       without server; add the default one */
+                    wuid = xs_fmt("%s@%s", v, def_srv);
+
+                    snac_debug(snac, 2, xs_fmt("mention without server '%s' '%s'", v, wuid));
+                }
+                else
+                    wuid = xs_dup(v);
+
                 /* query the webfinger about this fellow */
-                xs *v2    = xs_strip_chars_i(xs_dup(v), "@.");
                 xs *actor = NULL;
                 xs *uid   = NULL;
                 int status;
 
-                status = webfinger_request(v2, &actor, &uid);
+                status = webfinger_request(wuid, &actor, &uid);
 
                 if (valid_status(status)) {
                     xs *d = xs_dict_new();
                     xs *n = xs_fmt("@%s", uid);
-                    xs *l = xs_fmt("<a href=\"%s\" class=\"u-url mention\">%s</a>", actor, n);
 
                     d = xs_dict_append(d, "type",   "Mention");
                     d = xs_dict_append(d, "href",   actor);
@@ -392,11 +429,12 @@ void process_tags(snac *snac, const char *content, d_char **n_content, d_char **
 
                     tl = xs_list_append(tl, d);
 
-                    /* add the code */
-                    nc = xs_str_cat(nc, l);
+                    link = xs_fmt("<a href=\"%s\" class=\"u-url mention\">%s</a>", actor, n);
                 }
+
+                if (!xs_is_null(link))
+                    nc = xs_str_cat(nc, link);
                 else
-                    /* store as is */
                     nc = xs_str_cat(nc, v);
             }
             else
@@ -673,7 +711,7 @@ xs_dict *msg_follow(snac *snac, const char *q)
     if (xs_startswith(url_or_uid, "https:/"))
         actor = xs_dup(url_or_uid);
     else
-    if (!valid_status(webfinger_request(url_or_uid, &actor, NULL))) {
+    if (!valid_status(webfinger_request(url_or_uid, &actor, NULL)) || actor == NULL) {
         snac_log(snac, xs_fmt("cannot resolve user %s to follow", url_or_uid));
         return NULL;
     }
@@ -710,7 +748,7 @@ xs_dict *msg_note(snac *snac, const xs_str *content, const xs_val *rcpts,
     xs *to   = NULL;
     xs *cc   = xs_list_new();
     xs *irt  = NULL;
-    xs *tag  = NULL;
+    xs *tag  = xs_list_new();
     xs *atls = NULL;
     xs_dict *msg = msg_base(snac, "Note", id, NULL, "@now", NULL);
     xs_list *p;
@@ -730,12 +768,6 @@ xs_dict *msg_note(snac *snac, const xs_str *content, const xs_val *rcpts,
     /* format the content */
     fc2 = not_really_markdown(content);
 
-    /* extract the tags */
-    process_tags(snac, fc2, &fc1, &tag);
-
-    if (tag == NULL)
-        tag = xs_list_new();
-
     if (in_reply_to != NULL && *in_reply_to) {
         xs *p_msg = NULL;
         xs *wrk   = NULL;
@@ -778,6 +810,9 @@ xs_dict *msg_note(snac *snac, const xs_str *content, const xs_val *rcpts,
     else
         irt = xs_val_new(XSTYPE_NULL);
 
+    /* extract the mentions and hashtags and convert the content */
+    process_tags(snac, fc2, &fc1, &tag);
+
     /* create the attachment list, if there are any */
     if (!xs_is_null(attach)) {
         atls = xs_list_new();
