@@ -169,7 +169,7 @@ d_char *not_really_markdown(const char *content)
 
 const char *valid_tags[] = {
     "a", "p", "br", "br/", "blockquote", "ul", "li",
-    "span", "i", "b", "pre", "code", "em", "strong", NULL
+    "span", "i", "b", "u", "pre", "code", "em", "strong", NULL
 };
 
 d_char *sanitize(const char *content)
@@ -184,6 +184,7 @@ d_char *sanitize(const char *content)
 
     p = sl;
 
+    n = 0;
     while (xs_list_iter(&p, &v)) {
         if (n & 0x1) {
             xs *s1  = xs_strip_i(xs_crop_i(xs_dup(v), v[1] == '/' ? 2 : 1, -1));
