@@ -8,12 +8,15 @@
 #include "xs_socket.h"
 #include "xs_httpd.h"
 #include "xs_mime.h"
+#include "xs_time.h"
 
 #include "snac.h"
 
 #include <setjmp.h>
 #include <pthread.h>
 #include <semaphore.h>
+#include <fcntl.h>
+#include <stdint.h>
 
 #include <sys/resource.h> // for getrlimit()
 
@@ -299,7 +302,7 @@ void term_handler(int s)
 static pthread_mutex_t job_mutex;
 
 /* semaphre to trigger job processing */
-static sem_t job_sem;
+static sem_t *job_sem;
 
 /* fifo of jobs */
 xs_list *job_fifo = NULL;
@@ -332,7 +335,7 @@ void job_post(const xs_val *job, int urgent)
     }
 
     /* ask for someone to attend it */
-    sem_post(&job_sem);
+    sem_post(job_sem);
 }
 
 
@@ -341,7 +344,7 @@ void job_wait(xs_val **job)
 {
     *job = NULL;
 
-    if (sem_wait(&job_sem) == 0) {
+    if (sem_wait(job_sem) == 0) {
         /* lock the mutex */
         pthread_mutex_lock(&job_mutex);
 
@@ -362,7 +365,7 @@ void job_wait(xs_val **job)
 static void *job_thread(void *arg)
 /* job thread */
 {
-    int pid = (char *) arg - (char *) 0x0;
+    int pid = (int)(uintptr_t)arg;
 
     srv_debug(1, xs_fmt("job thread %d started", pid));
 
@@ -478,6 +481,8 @@ void httpd(void)
     pthread_t threads[MAX_THREADS] = {0};
     int n_threads = 0;
     int n;
+    time_t start_time = time(NULL);
+    char sem_name[24];
 
     address = xs_dict_get(srv_config, "address");
     port    = xs_number_get(xs_dict_get(srv_config, "port"));
@@ -503,7 +508,8 @@ void httpd(void)
 
     /* initialize the job control engine */
     pthread_mutex_init(&job_mutex, NULL);
-    sem_init(&job_sem, 0, 0);
+    sprintf(sem_name, "/job_%d", getpid());
+    job_sem = sem_open(sem_name, O_CREAT, 0644, 0);
     job_fifo = xs_list_new();
 
     /* initialize sleep control */
@@ -562,5 +568,10 @@ void httpd(void)
     job_fifo = xs_free(job_fifo);
     pthread_mutex_unlock(&job_mutex);
 
-    srv_log(xs_fmt("httpd stop %s:%d", address, port));
+    sem_close(job_sem);
+    sem_unlink(sem_name);
+
+    xs *uptime = xs_str_time_diff(time(NULL) - start_time);
+
+    srv_log(xs_fmt("httpd stop %s:%d (run time: %s)", address, port, uptime));
 }
