@@ -30,10 +30,7 @@ int webfinger_request(const char *qs, char **actor, char **user)
     }
     else {
         /* it's a user */
-        xs *s = xs_dup(qs);
-
-        if (xs_startswith(s, "@"))
-            s = xs_crop_i(s, 1, 0);
+        xs *s = xs_strip_chars_i(xs_dup(qs), "@.");
 
         l = xs_split_n(s, "@", 1);
 
@@ -46,7 +43,8 @@ int webfinger_request(const char *qs, char **actor, char **user)
     if (host == NULL || resource == NULL)
         return 400;
 
-    headers = xs_dict_append(headers, "accept", "application/json");
+    headers = xs_dict_append(headers, "accept",     "application/json");
+    headers = xs_dict_append(headers, "user-agent", USER_AGENT);
 
     /* is it a query about one of us? */
     if (strcmp(host, xs_dict_get(srv_config, "host")) == 0) {
@@ -85,7 +83,8 @@ int webfinger_request(const char *qs, char **actor, char **user)
                 if (xs_type(v) == XSTYPE_DICT) {
                     char *type = xs_dict_get(v, "type");
 
-                    if (type && strcmp(type, "application/activity+json") == 0) {
+                    if (type && (strcmp(type, "application/activity+json") == 0 ||
+                                strcmp(type, "application/ld+json; profile=\"https://www.w3.org/ns/activitystreams\"") == 0)) {
                         *actor = xs_dup(xs_dict_get(v, "href"));
                         break;
                     }
